/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.token;

import java.io.BufferedReader;
import java.io.IOException;
import java.net.URLDecoder;
import java.net.URLEncoder;

import org.opengion.fukurou.util.LogWriter;
import org.opengion.fukurou.util.StringUtil;
import org.opengion.fukurou.util.URLConnect;
import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.html.AbstractCreateToken;

import aspera.tokengen.ASTokenGen;
/**
 * Asperaのトークンを付加するためのプラグインです。
 * 
 * 毎回tokengenをcallするため利用する場合は若干時間がかかります。
 * 
 * 
 * 以下をシステムリソースで設定可能です
 * AS_NODE_USER		: ノードユーザ。初期値はnode_user
 * AS_NODE_PASS		: ノードユーザのパスワード。初期値はnode_pass
 * AS_NODE_URL		: ノードのアドレス。初期値はhttp://devdemo.asperasoft.com:9092
 * 
 * tokenを利用した認証についてはAsperaの開発者サイト
 * https://developer.asperasoft.com/Home
 *  Home
 *  >  GENERAL REFERENCE
 *  >  Authentication and authorization in FASP transfers
 *  >  Authorization
 * を参照してください。
 * 
 * Home
 * >  WEB APIs
 * >  Node
 * >  v1
 * >  Rest API - Transfer Operations
 * でNodeAPIについての解説があります。
 *
 * @og.group 画面表示
 * @og.rev 5.8.2.2 (2014/12/19)
 *
 * @version  5.0
 * @author	 Takahashi Masakazu
 * @since    JDK5.0,
 */
public class CreateToken_Aspera extends AbstractCreateToken {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "5.8.2.1 (2014/12/13)" ;
	
	private static final String AS_NODE_USER	 = StringUtil.nval(HybsSystem.sys( "AS_TOKEN_USER " ),"node_user" );
	private static final String AS_NODE_PASS	 = StringUtil.nval(HybsSystem.sys( "AS_TOKEN_USER " ),"node_pass" );
	private static final String AS_NODE_URL	 = StringUtil.nval(HybsSystem.sys( "AS_NODE_URL " ),"http://devdemo.asperasoft.com:9092" );
	
	private static String AS_NODE_API = AS_NODE_URL + "/files/download_setup";
	private URLConnect conn			= null;
	
//	private ASTokenGen generator; // static化するとセキュリティ上危険な可能性があるのでやめておく(transferPathの削除が出来ないっぽい）
	
	/**
	 * デフォルトコンストラクター。
	 *
	 */
	public CreateToken_Aspera(){
//	トークンを作成する場合
//		generator = new ASTokenGen( AS_NODE_USER, null, Direction.DOWNLOAD);
//		generator.setAsTokenGenPath( AS_TOKENGEN_PATH );
//		generator.setEncryptionKey( AS_TOKEN_ENCR );
		
		conn = new URLConnect(AS_NODE_API ,AS_NODE_USER +":"+AS_NODE_PASS);
		conn.setRequestProperty( "Content-Type,Accept", "application/json; charset=UTF8,application/json" );
	}


	/**
	 * アスペラのトークンを付けたURLを返します。
	 *
	 *
	 * @return  URL
	 */
	@Override
	public String  generateURL( final String inURL, final long time, final String user, final String[] param ){
//		tokenGeneratorでのトークン作成方法
//		generator.addTransferPath( inURL ); 
//      generator.setLifeSeconds(3600);
//        ASToken token = generator.generateASToken();
//      if( token == null ){
//      	System.out.println("Aspera token is null : " + inURL);
//      	return inURL;
//      }
//		token.toString();
        
        // AsperaNodeAPIを利用してトークン付JSONを取得する
        // ユーザはトークン用の共通ユーザを利用してconnectする。
		StringBuffer rtnMessage = new StringBuffer("");
        try{
    		String encURL = URLEncoder.encode( inURL, "UTF-8" );
    		//String encURL = inURL;
            conn.setPostData( "{ \"transfer_requests\" : [ { \"transfer_request\" : { \"paths\" : [ { \"source\" : \""+encURL+"\" } ] } } ] }" );

			conn.connect();
			
			BufferedReader reader = conn.getReader();
            String s;
            while ((s = reader.readLine()) != null) {
            	rtnMessage.append( URLDecoder.decode( s, "UTF-8" ) );
            }
            reader.close();

        }
        catch( IOException ex ) {
        	System.out.println(ex);
			String errMsg = "Daemon_URLConnect:データ取得中にエラーが発生しました。" + HybsSystem.CR
						+ " inURL=[" + inURL + "]" + HybsSystem.CR
						+ ex;
			LogWriter.log( errMsg );
			rtnMessage = new StringBuffer(inURL);
		}
        catch( Exception ex){
        	System.out.println(ex);
			String errMsg = "Daemon_URLConnect:データ取得中にエラーが発生しました。" + HybsSystem.CR
						+ " inURL=[" + inURL + "]" + HybsSystem.CR
						+ ex;
			LogWriter.log( errMsg );
			rtnMessage = new StringBuffer(inURL);
        }
		finally {
			if( conn != null ) { conn.disconnect(); }
		}
        
		// エラー時も無視してそのままJSON形式を返却する。
		// aspera用のJavaScripptを画面に読み込ませておく必要あり。
		
		
		return "javascript:fileControls.selectFolder(" + rtnMessage.toString().replace( "\"", "&quot;" ) + ")";
	}	

}
